# Default VM settings for various OSs
#
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 

from .nics import XenNic
from .nics import KvmNic
from .VMActions import VMActions
from .VMNetWare import VMNetWare
from .VMRedHat import VMRedHat
from .VMSUSE import VMSUSE
from .VMUbuntu import VMUbuntu
from .VMWindows import VMWindows
from .VMReaR import VMReaR
from vmdisks import Disk
from .VMPXE import VMPXE
from . import caps
from . import msg


class GenericDefaults:
    def group(self):
        return 'other'
    def actions(self):
        return VMActions()
    def install_sources(self):
        return ('generic', ['dev:/'])
    def name(self):
        return msg.other_os
    def virt_support(self):
        return ('xen', 'hvm')
    def apic(self):
        return False
    def max_cpus(self):
        if caps.is_kvm() or caps.is_qemu():
            return 255
        else:
            return 255
    def default_memory(self):
        return 512
    def min_memory(self):
        return 4
    def max_memory(self):
        #if caps.is_x86_64_capable():
        #    mb = 128*1024  # Arbitrarily large...
        #else:
        #    if caps.is_pae_capable():
        #        mb = 16*1024  # PAE limits at 64GB; Xen has lower per-vm limits
        #    else:
        #        mb = 2*1024  # hardware limit is 4GB; Xen has lower per-vm limits
        mb = self.default_memory()
        return mb
    def _disk_template(self, num, sizeGB):
        return Disk(pdev=None, vdev=num, blocks=sizeGB*1024*1024*2, vdevType=Disk.DEVICE_DISK, virtio=self.virtio())
    def _cdrom_template(self, num):
        return Disk(pdev=None, vdev=num, vdevType=Disk.DEVICE_CDROM, virtio=self.virtio())
    def _disk_size(self):
        return 4
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic() ]
        else:
            return [ XenNic() ]
    def localtime(self):
        return False
    def xterm_args(self):
        return ['+u8', '-fg', 'white', '-bg', 'black']
    def auto_install(self):
        return None
    def num_vcpus(self):
        return 1
    def boot_pxe(self):
        return False
    def supports_32nonpae(self):
        return True
    def is_upgradeable(self):
        return False
    def hpet(self):
        if caps.is_kvm() or caps.is_qemu():
            # On by default for KVM
            return True
        else:
            return False
    def usb(self):
        return False
    def virtio(self):
        # KVM only
        return False
    def supports_direct_kernel_boot(self):
        return False
    def direct_kernel_boot(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def sound(self):
        return None
    def multi_stage_install(self):
        return 0

class LinuxDefaults(GenericDefaults):
    def xterm_args(self):
        # Linux behaves much better with an 80x24 xterm.
        args = GenericDefaults.xterm_args(self)
        args.extend(['-geometry', '80x24'])
        return args

class RedHatDefaults(LinuxDefaults):
    def group(self):
        return 'redhat'
    def name(self):
        return msg.redhat_other
    def install_sources(self):
        return ['redhat', ('ftp://', 'http://', 'nfs://', 'dev:/')]
    def actions(self):
        return VMRedHat()
    def virt_support(self):
        return ('hvm', 'xen')
    def min_memory(self):
        # Minimum for text-based install
        # As per http://fedora.redhat.com/docs/release-notes/
        return 128
    def default_memory(self):
        # FIXME:  how do I know if install source is 32 or 64 bit?
        #if caps.is_x86_64_capable():
        #    return 512
        #else:
        return 256
    def auto_install(self):
        return msg.kickstart_desc
    def apic(self):
        # RHEL3 64bit needs this.  Slows down 32bit:(
        return True
    def usb(self):
        return True

# https://www.redhat.com/rhel/details/limits/
class RHEL3Defaults(RedHatDefaults):
    def name(self):
        return msg.rhel3
    def virt_support(self):
        return ('hvm')
    def max_cpus(self):
        return 16
    def min_memory(self):
        return 256
    def max_memory(self):
        return self.default_memory()
        #return 8*1024

class RHEL4Defaults(RedHatDefaults):
    def name(self):
        return msg.rhel4
    def virt_support(self):
        # RHEL4 doesn't support para, but RHEL4 update 8 does, so prefer hvm.
        return ('hvm', 'xen')
    def min_memory(self):
        return 256
    def max_memory(self):
        return self.default_memory()
        #return 16*1024
    def _disk_size(self):
        return 8
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def num_vcpus(self):
        return caps.vcpu_processors(4)

class RHEL5Defaults(RedHatDefaults):
    def name(self):
        return msg.rhel5
    def virt_support(self):
        return ('hvm', 'xen')
    def min_memory(self):
        return 512
    def max_memory(self):
        return self.default_memory()
        #return 16*1024
    def _disk_size(self):
        return 8
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def num_vcpus(self):
        return caps.vcpu_processors(1)

class RHEL6Defaults(RedHatDefaults):
    def name(self):
        return msg.rhel6
    def virt_support(self):
        return ('hvm', 'xen')
    def min_memory(self):
        return 512
    def max_memory(self):
        return self.default_memory()
        #return 16*1024
    def _disk_size(self):
        return 8
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def num_vcpus(self):
        return caps.vcpu_processors(1)
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]

class RHEL7Defaults(RHEL6Defaults):
    def name(self):
        return msg.rhel7

class SUSEDefaults(LinuxDefaults):
    def group(self):
        return 'suse'
    def name(self):
        return msg.suse_other
    def virt_support(self):
        return ('xen', 'hvm')
    def install_sources(self):
        return ['suse', ('ftp://', 'http://', 'nfs://', 'smb://', 'dev:/')]
    def actions(self):
        return VMSUSE()
    def min_memory(self):
        # Minimum for text-based install
        return 192
    def default_memory(self):
        return 256
    def auto_install(self):
        return msg.autoyast_desc

class openSUSEDefaults(SUSEDefaults):
    def name(self):
        return msg.opensuse
    def default_memory(self):
        return 384
    def _disk_size(self):
        return 8
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def sound(self):
        if caps.is_xen():
            return "sb16"
        else:
            return "ac97"

class openSUSE11Defaults(SUSEDefaults):
    def name(self):
        return msg.opensuse11
    def default_memory(self):
        return 512
    def _disk_size(self):
        return 8
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def supports_32nonpae(self):
        return False
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]
    def sound(self):
        if caps.is_xen():
            return "sb16"
        else:
            return "ac97"
    def supports_direct_kernel_boot(self):
        return True
    def direct_kernel_boot(self):
        return True

class openSUSE12Defaults(openSUSE11Defaults):
    def name(self):
        return msg.opensuse12
    def default_memory(self):
        return 768

class openSUSE13Defaults(openSUSE12Defaults):
    def name(self):
        return msg.opensuse13
    def default_memory(self):
        return 768

class openSUSE42Defaults(openSUSE13Defaults):
    def name(self):
        return msg.opensuse42
    def default_memory(self):
        return 1024

class openSUSE15Defaults(openSUSE42Defaults):
    def name(self):
        return msg.opensuse15
    def default_memory(self):
        return 1024
    def _disk_size(self):
        return 12

class openSUSETumbleweedDefaults(openSUSE15Defaults):
    def name(self):
        return msg.opensuse_tumbleweed
    def default_memory(self):
        return 1024
    def _disk_size(self):
        return 16

class SLED10Defaults(SUSEDefaults):
    def name(self):
        return msg.sled10
    def min_memory(self):
        return 256
    def default_memory(self):
        return 512
    def _disk_size(self):
        return 8
    def is_upgradeable(self):
        return True
    def apic(self):
        # Turned on even for uni-processor
        return True
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]
    def sound(self):
        if caps.is_xen():
            return "sb16"
        else:
            return "ac97"
    def supports_direct_kernel_boot(self):
        return True
    def direct_kernel_boot(self):
        return True

class SLED11Defaults(SUSEDefaults):
    def name(self):
        return msg.sled11
    def min_memory(self):
        return 256
    def default_memory(self):
        return 512
    def _disk_size(self):
        return 16
    def supports_32nonpae(self):
        return False
    def apic(self):
        # Turned on even for uni-processor
        return True
    def is_upgradeable(self):
        return True
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]
    def sound(self):
        if caps.is_xen():
            return "sb16"
        else:
            return "ac97"
    def supports_direct_kernel_boot(self):
        return True
    def direct_kernel_boot(self):
        return True

class SLED12Defaults(SUSEDefaults):
    def name(self):
        return msg.sled12
    def min_memory(self):
        return 256
    def default_memory(self):
        return 1024
    def _disk_size(self):
        return 16
    def supports_32nonpae(self):
        return False
    def apic(self):
        # Turned on even for uni-processor
        return True
    def is_upgradeable(self):
        return True
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]
    def sound(self):
        if caps.is_xen():
            return "sb16"
        else:
            return "ac97"
    def supports_direct_kernel_boot(self):
        return True
    def direct_kernel_boot(self):
        return True

class SLED15Defaults(SLED12Defaults):
    def name(self):
        return msg.sled15

class SLES8Defaults(SUSEDefaults):
    def name(self):
        return msg.sles8
    def virt_support(self):
        return ('hvm')

class SLES9Defaults(SUSEDefaults):
    def name(self):
        return msg.sles9
    def default_memory(self):
        return 512
    def virt_support(self):
        # SLES9 supports para starting with SP4
        return ('hvm', 'xen')
    def apic(self):
        # Turned on even for uni-processor
        return True
    def num_vcpus(self):
        return caps.vcpu_processors(2)
    def usb(self):
        return True

class SLES10Defaults(SUSEDefaults):
    def name(self):
        return msg.sles10
    def default_memory(self):
        return 512
    def num_vcpus(self):
        return caps.vcpu_processors(4)
    def apic(self):
        return True
    def is_upgradeable(self):
        return True
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]
    def supports_direct_kernel_boot(self):
        return True
    def direct_kernel_boot(self):
        return True

class SLES11Defaults(SUSEDefaults):
    def name(self):
        return msg.sles11
    def default_memory(self):
        return 512
    def _disk_size(self):
        return 8
    def num_vcpus(self):
        return caps.vcpu_processors(4)
    def apic(self):
        return True
    def is_upgradeable(self):
        return True
    def supports_32nonpae(self):
        return False
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]
    def supports_direct_kernel_boot(self):
        return True
    def direct_kernel_boot(self):
        return True

class SLES12Defaults(SUSEDefaults):
    def name(self):
        return msg.sles12
    def default_memory(self):
        return 1024
    def _disk_size(self):
        return 12
    def num_vcpus(self):
        return caps.vcpu_processors(4)
    def apic(self):
        return True
    def is_upgradeable(self):
        return True
    def supports_32nonpae(self):
        return False
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]
    def supports_direct_kernel_boot(self):
        return True
    def direct_kernel_boot(self):
        return True

class SLES15Defaults(SLES12Defaults):
    def name(self):
        return msg.sles15
    def _disk_size(self):
        return 12

class UbuntuDefaults(LinuxDefaults):
    def group(self):
        return 'ubuntu'
    def name(self):
        return msg.ubuntu_other
    def virt_support(self):
        return ('hvm')
    def actions(self):
        return VMUbuntu()
    def virt_support(self):
        return ('hvm')
    def min_memory(self):
        return 128
    #def default_memory(self):
    #    return 256
    def auto_install(self):
        return msg.kickstart_desc
    def apic(self):
        return True
    def usb(self):
        return True

class Ubuntu10Defaults(UbuntuDefaults):
    def name(self):
        return msg.ubuntu10
    def virt_support(self):
        return ('hvm')
    def max_cpus(self):
        return 16
    def min_memory(self):
        return 256
    def max_memory(self):
        return self.default_memory()
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]

class Ubuntu11Defaults(UbuntuDefaults):
    def name(self):
        return msg.ubuntu11
    def virt_support(self):
        return ('hvm')
    def max_cpus(self):
        return 16
    def min_memory(self):
        return 512
    def max_memory(self):
        return self.default_memory()
    def virtio(self):
        if caps.is_kvm() or caps.is_qemu():
            return True
        else:
            return False
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='virtio') ]
        else:
            return [ XenNic(model='para') ]

class Ubuntu12Defaults(Ubuntu11Defaults):
    def name(self):
        return msg.ubuntu12

class NetWareDefaults(GenericDefaults):
    def group(self):
        return 'netware'
    def name(self):
        return msg.nw65_older
    def virt_support(self):
        return ('hvm')
    def actions(self):
        return VMNetWare()
    def min_memory(self):
        return 32
    def default_memory(self):
        return 768
    def localtime(self):
        return True
    def xterm_args(self):
        # NetWare behaves much better with an 80x25 xterm.
        args = GenericDefaults.xterm_args(self)
        args.extend(['-geometry', '80x25'])
        return args
    def auto_install(self):
        return msg.nwresponse_desc
    def num_vcpus(self):
        if caps.is_kvm() or caps.is_qemu():
            return 1
        else:
            return caps.vcpu_processors(2)
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='e1000') ]
        else:
            return [ XenNic() ]
    def _disk_template(self, num, sizeGB):
        diskargs = { "sparse" : False }
        return Disk(pdev=None, vdev=num, blocks=sizeGB*1024*1024*2, vdevType=Disk.DEVICE_DISK, virtio=self.virtio(), args=diskargs)

class NOES2NWDefaults(NetWareDefaults):
    def name(self):
        return msg.oes2nw
    def virt_support(self):
        return ('xen')
    def min_memory(self):
        return 512
    def default_memory(self):
        return 1024
    def _disk_size(self):
        # Docs say minimum 6, recommended 10
        return 10
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]

class NOES2LDefaults(SUSEDefaults):
    def name(self):
        return msg.oes2l
    def min_memory(self):
        return 512
    def default_memory(self):
        return 512
    def _disk_size(self):
        # Docs say minimum 6, recommended 10
        return 10
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def num_vcpus(self):
        return caps.vcpu_processors(4)
    def is_upgradeable(self):
        return True

class NOES11Defaults(SUSEDefaults):
    def name(self):
        return msg.oes11
    def min_memory(self):
        return 512
    def default_memory(self):
        return 512
    def _disk_size(self):
        # Docs say minimum 6, recommended 10
        return 10
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def num_vcpus(self):
        return caps.vcpu_processors(4)
    def is_upgradeable(self):
        return True

class WindowsDefaults(GenericDefaults):
    def group(self):
        return 'windows'
    def virt_support(self):
        return ('hvm')
    def actions(self):
        return VMWindows()
    def localtime(self):
        return True
    def _disk_size(self):
        return 8
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def min_memory(self):
        return 64
    def default_memory(self):
        return 256
    def usb(self):
        return True
    def hpet(self):
        return False
class WindowsNTDefaults(WindowsDefaults):
    def name(self):
        return msg.windows_nt
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def min_memory(self):
        return 16
    def default_memory(self):
        return 128
    def max_memory(self):
        # WARNING:  NT has installer bug; crashes if > 256 MB RAM; can
        #           manually increase post-install.
        return self.default_memory()
        #return 256
class WindowsOtherDefaults(WindowsDefaults):
    def name(self):
        return msg.windows_other
class WindowsOtherx64Defaults(WindowsDefaults):
    def name(self):
        return msg.windows_other_x64
    def apic(self):
        return True
    def num_vcpus(self):
        return caps.vcpu_processors(2)
class WindowsXPDefaults(WindowsDefaults):
    def name(self):
        return msg.windows_xp
    def default_memory(self):
        return 512
    def num_vcpus(self):
        return caps.vcpu_processors(2)
    def apic(self):
        return True
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='e1000') ]
        else:
            return [ XenNic(model='rtl8139') ]
    def sound(self):
        if caps.is_xen():
            return "sb16"
        else:
            return "ac97"
    def multi_stage_install(self):
        return 1
class WindowsXPx64Defaults(WindowsDefaults):
    def name(self):
        return msg.windows_xp_x64
    def default_memory(self):
        return 512
    def num_vcpus(self):
        return caps.vcpu_processors(2)
    def apic(self):
        return True
    def multi_stage_install(self):
        return 1
class WindowsVistaDefaults(WindowsDefaults):
    def name(self):
        return msg.windows_vista
    def min_memory(self):
        return 512
    def default_memory(self):
        return 1024
    def _disk_size(self):
        return 30
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='e1000') ]
        else:
            return [ XenNic(model='rtl8139') ]
    def num_vcpus(self):
        return caps.vcpu_processors(2)
    def apic(self):
        return True
    def sound(self):
        if caps.is_xen():
            return "sb16"
        else:
            return "ac97"
    def multi_stage_install(self):
        return 1
class WindowsVistax64Defaults(WindowsVistaDefaults):
    def name(self):
        return msg.windows_vista_x64
class Windows7Defaults(WindowsVistaDefaults):
    def name(self):
        return msg.windows_7
    def min_memory(self):
        return 1024
class Windows7x64Defaults(Windows7Defaults):
    def name(self):
        return msg.windows_7_x64
class Windows8Defaults(Windows7Defaults):
    def name(self):
        return msg.windows_8
class Windows8x64Defaults(Windows8Defaults):
    def name(self):
        return msg.windows_8_x64
class Windows10Defaults(Windows8Defaults):
    def name(self):
        return msg.windows_10
class Windows10x64Defaults(Windows10Defaults):
    def name(self):
        return msg.windows_10_x64
class WindowsServer2008Defaults(WindowsDefaults):
    def name(self):
        return msg.windows_server_2008
    def virt_support(self):
        # 'hvm-ext' here means enlightened windows - AKA, Viridian
        return ('hvm-ext', 'hvm')
    def min_memory(self):
        return 512
    def default_memory(self):
        return 1024
    def _disk_size(self):
        return 40
    def disks(self):
        return [ self._disk_template(0, self._disk_size()) ]
    def nics(self):
        if caps.is_kvm() or caps.is_qemu():
            return [ KvmNic(model='e1000') ]
        else:
            return [ XenNic(model='rtl8139') ]
    def enlightened(self):
        return True
    def num_vcpus(self):
        return caps.vcpu_processors(4)
    def apic(self):
        return True
    def multi_stage_install(self):
        return 2
class WindowsServer2008x64Defaults(WindowsServer2008Defaults):
    def name(self):
        return msg.windows_server_2008_x64
    def hpet(self):
        if caps.is_xen():
            return True
        else:
            return False
class WindowsServer2012Defaults(WindowsServer2008x64Defaults):
    def name(self):
        return msg.windows_server_2012
    def multi_stage_install(self):
        return 1
class WindowsServer2016Defaults(WindowsServer2008x64Defaults):
    def name(self):
        return msg.windows_server_2016
    def multi_stage_install(self):
        return 1
class WindowsServer2019Defaults(WindowsServer2008x64Defaults):
    def name(self):
        return msg.windows_server_2019
    def multi_stage_install(self):
        return 1

class Solaris9Defaults(GenericDefaults):
    def group(self):
        return 'solaris'
    def name(self):
        return msg.solaris9
    def virt_support(self):
        return ('hvm')
    # http://www.sun.com/software/solaris/9/specs.xml
    def min_memory(self):
        return 64
    def default_memory(self):
        return 128
    def num_vcpus(self):
        return caps.vcpu_processors(4)
    def apic(self):
        return True
# FIXME:  learn about this...
#    def auto_install(self):
#        return "JumpStart"
class Solaris10Defaults(Solaris9Defaults):
    def name(self):
        return msg.solaris10
    # http://www.sun.com/software/solaris/specs.jsp
    def min_memory(self):
        return 256
    def default_memory(self):
        return 512
    def virt_support(self):
        # FIXME:  need to code support for bootstrapping PV Solaris!
        # http://www.opensolaris.org/os/community/xen/
        return ('hvm', 'xen')

class PXEDefaults(LinuxDefaults):
    def group(self):
        return 'other'
    def name(self):
        return msg.type_pxe
    def virt_support(self):
        return ('xen', 'hvm')
    def install_sources(self):
        return ['pxe', ('ftp://', 'http://', 'nfs://', 'smb://', 'dev:/, pxe')]
    def actions(self):
        return VMPXE()
    def min_memory(self):
        # Minimum for text-based install
        return 192
    def default_memory(self):
        return 256
    def auto_install(self):
        return msg.autoyast_desc

class ReaRDefaults(LinuxDefaults):
    def group(self):
        return 'other'
    def name(self):
        return msg.type_rear
    def virt_support(self):
        return ('xen', 'hvm')
    def actions(self):
        return VMReaR()
    def min_memory(self):
        return 256
    def default_memory(self):
        return 512
    def disks(self):
        return []
    def install_sources(self):
        return ('rear', ['dev:/'])

VMDefaults = {
    'oes2l'    : NOES2LDefaults(),
    'oes11'    : NOES11Defaults(),
    'oes2nw'   : NOES2NWDefaults(),
    'opensuse' : openSUSEDefaults(),
    'opensuse11' : openSUSE11Defaults(),
    'opensuse12' : openSUSE12Defaults(),
    'opensuse13' : openSUSE13Defaults(),
    'opensuse42' : openSUSE42Defaults(),
    'opensuse15' : openSUSE15Defaults(),
    'opensuse_tumbleweed' : openSUSETumbleweedDefaults(),
    'other'    : GenericDefaults(),
    'redhat'   : RedHatDefaults(),
    'rhel3'    : RHEL3Defaults(),
    'rhel4'    : RHEL4Defaults(),
    'rhel5'    : RHEL5Defaults(),
    'rhel6'    : RHEL6Defaults(),
    'rhel7'    : RHEL7Defaults(),
    'sled10'   : SLED10Defaults(),
    'sled11'   : SLED11Defaults(),
    'sled12'   : SLED12Defaults(),
    'sled15'   : SLED15Defaults(),
    'sles10'   : SLES10Defaults(),
    'sles11'   : SLES11Defaults(),
    'sles12'   : SLES12Defaults(),
    'sles15'   : SLES15Defaults(),
    'sles8'    : SLES8Defaults(),
    'sles9'    : SLES9Defaults(),
    'suse'     : SUSEDefaults(),
    'solaris10': Solaris10Defaults(),
    'solaris9' : Solaris9Defaults(),
    'ubuntu'   : UbuntuDefaults(),
    'ubuntu10' : Ubuntu10Defaults(),
    'ubuntu11' : Ubuntu11Defaults(),
    'ubuntu12' : Ubuntu12Defaults(),
    'winserver2019'   : WindowsServer2019Defaults(),
    'winserver2016'   : WindowsServer2016Defaults(),
    'winserver2012'   : WindowsServer2012Defaults(),
    'winserver2008'   : WindowsServer2008Defaults(),
    'winserver2008x64': WindowsServer2008x64Defaults(),
    'windows10'       : Windows10Defaults(),
    'windows10x64'    : Windows10x64Defaults(),
    'windows8'        : Windows8Defaults(),
    'windows8x64'     : Windows8x64Defaults(),
    'windows7'        : Windows7Defaults(),
    'windows7x64'     : Windows7x64Defaults(),
    'windowsvista'    : WindowsVistaDefaults(),
    'windowsvistax64' : WindowsVistax64Defaults(),
    'windows'         : WindowsOtherDefaults(),
    'windowsx64'      : WindowsOtherx64Defaults(),
    'windowsxp'       : WindowsXPDefaults(),
    'windowsxpx64'    : WindowsXPx64Defaults(),
    'windowsnt'       : WindowsNTDefaults(),
    'pxe'             : PXEDefaults(),
    'rear'            : ReaRDefaults(),
}

default_os_type = caps.get_os_type()
