# Common code for all kvm guests
#
# Initially derived from virt-inst, Copyright 2005-2006 Red Hat, Inc.
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 

import os
import time

from .exceptions import *
from .log import log
from . import paths
from . import util
import vmdisks
#import xen_hypervisor
from . import hypervisor
from . import libvirt_hypervisor
from . import keytable
from . import caps
from .guest import Guest
from vmdisks import Disk
from .direct_kernel_boot import DirectKernelBoot

class KVMGuest(Guest):
#    def _get_name_xen(self):
#        return 'name="%s"\n' % self.options.name

    def _get_memory_xml(self):
        # XML wants KB
        return ['<memory>%d</memory>' % (self.options.max_memoryMB*1024,),
                '<currentMemory>%d</currentMemory>' % (self.options.memoryMB*1024,)]

    def _get_uuid_xml(self):
        return ['<uuid>%s</uuid>' % self.options.uuid]

    def _get_action(self, action):
        if self.isInstall:
            return 'destroy'
        else:
            if action == 'crash': return self.options.on_crash
            if action == 'poweroff': return self.options.on_poweroff
            if action == 'reboot': return self.options.on_reboot
    def _get_action_xml(self):
        return ['<on_poweroff>%s</on_poweroff>' % self._get_action('poweroff'),
                '<on_reboot>%s</on_reboot>' % self._get_action('reboot'),
                '<on_crash>%s</on_crash>' % self._get_action('crash')]

    def _get_vcpus_xml(self):
        return ['<vcpu>%d</vcpu>' % self.options.vcpus]

    def _get_cpumodel_xml(self):
        cpu = []
        if caps.kvm_arch() == 'aarch64':
            cpu = ["<cpu mode='host-model'>"]
            cpu.append("</cpu>")
        return cpu

    def _get_localtime_xml(self):
        clock = []
        hpet = None
        if self.options.hpet is False:
            hpet = "<timer name='hpet' present='no'/>"
        if self.os().localtime():
            clock = ["<clock offset='localtime'>"]
        else:
            clock = ["<clock offset='utc'>"]
        if hpet:
            clock.append(hpet)
        clock.append("</clock>")
        return clock

    def getCommonXml(self):
        s = []
        s.extend(self._get_name_xml())
        s.extend(self._get_uuid_xml())
        s.extend(self._get_memory_xml())
        s.extend(self._get_vcpus_xml())
        s.extend(self._get_cpumodel_xml())
#BFR not needed?        s.extend(self._get_keymap_xml())
        return s

    def _get_os_xml(self):
        os = []
        if self.isInstall:
            if self.options.pxe_boot == True:
                bootdev = 'network'
            else:
                if type(self.options.source) is Disk:
                    if self.options.source.vdevtype == Disk.DEVICE_DISK:
                        bootdev = 'hd'
                    else:
                        bootdev = 'cdrom'
                else:
                    bootdev = 'cdrom'
        else:
            bootdev = 'hd'

        if caps.kvm_arch() == 'aarch64':
            os.append("<type arch='%s' machine='virt'>hvm</type>" % caps.kvm_arch())
            os.append("<loader readonly='yes' type='pflash'>/usr/share/qemu/aavmf-aarch64-code.bin</loader>")
        elif caps.kvm_arch() == 's390x':
            os.append("<type arch='%s' machine='s390-ccw-virtio'>hvm</type>" % caps.kvm_arch())
        else:
            os.append("<type arch='%s' machine='pc'>hvm</type>" % caps.kvm_arch())
        if self.isInstall and self.options.direct_kernel_boot and self.dkb:
            os.append("<kernel>%s</kernel>" % self.dkb.kernel)
            os.append("<initrd>%s</initrd>" % self.dkb.initrd)
            os.append("<cmdline>%s %s</cmdline>" % (self.options._install_args, self.options.extra_args))
        os.append("<boot dev='%s'/>" % bootdev)
        return os 

    def _get_disk_xml(self):
        """Get the disk config in the libvirt XML format"""
        available_cdroms = [{'hda': 0}, {'hdb': 0}, {'hdc': 0}, {'hdd': 0}]
        ret = []
        saved_mode = None
        target_disk = self.options.disks[0]
        if self.isInstall and target_disk:
            index, saved_mode = target_disk.get_cache_mode()
            # The destination disk only during installation is this considered safe to do
            target_disk.set_cache_mode("unsafe")
        for disk in self.options.disks + self.tmpdisks:
            use_virtio = self.options.virtio and disk.virtio
            ret.extend(disk.get_xml_config(use_virtio, available_cdroms))
        if self.isInstall and target_disk:
            target_disk.set_cache_mode(saved_mode)
        return ret

    def _get_network_xml(self):
        """Get the network config in the libvirt XML format"""
        nics = []
        for n in self.options.nics:
            model = ""
            opts = []
            if caps.kvm_arch() == 's390x':
                nic = ["<interface type='direct'>"]
            elif n.bridge:
                nic = ["<interface type='bridge'>"]
                opts.append("<source bridge='%s'/>" % n.bridge)
            else:
                nic = ["<interface type='user'>"]
            if n.mac:
                opts.append("<mac address='%s'/>" % n.mac)
            if caps.kvm_arch() == 's390x':
                ethname = None
                if n.bridge:
                    if n.bridge.find('@') != -1:
                        ethname, ignore = n.bridge.rsplit("@",1)
                    if not ethname:
                        ethname = n.bridge
                else:
                    ethname = 'eth0'
                opts.append("<source dev='%s' mode='bridge'/>" % ethname)
            if n.model:
                opts.append("<model type='%s'/>" % n.model)
#BFR            opts.append("<script path='/etc/xen/qemu-ifup'/>")
            nic.append(opts)
            nic.append('</interface>')
            nics.append(nic)
        return nics

    def _get_serial_xml(self):
        """Get the serial config in the libvirt XML format"""
        serial = []
        if self.options.graphics == 'none':
            serial.append("<serial type='pty'>")
            target = ["<target port='0'/>"]
            serial.append(target)
            serial.append("</serial>")
            serial.append("<console type='pty'>")
            target = ["<target type='serial' port='0'/>"]
            serial.append(target)
            serial.append("</console>")
        return serial

    def _get_host_devices_xml(self):
        """Get the host device config in the libvirt XML format"""
        ##### Example PCI
        #<hostdev mode='subsystem' type='pci' managed='yes'>
        #  <source>
        #    <address domain='0x0000' bus='0x06' slot='0x03' function='0x0'/>
        #  </source>
        #</hostdev>
        ##### Example USB
        #<hostdev mode="subsystem" type="usb" managed="yes">
        #  <source>
        #    <vendor id="0x04f3"/>
        #    <product id="0x0213"/>
        #  </source>
        #</hostdev>
        if not self.options.host_devices:
            return ""
        host_devices = []
        for hd in self.options.host_devices:
            hdev = []
            src = []
            data = []
            managed = 'no'
            if hd.managed:
                managed = 'yes'
            if hd.type == 'pci':
                pci = hd.host_device
                hdev.append("<hostdev mode='subsystem' type='pci' managed='%s'>" % managed)
                src.append("<source>")
                data.append("<address domain='0x%s' bus='0x%s' slot='0x%s' function='0x%s'/>" % (pci.domain, pci.bus, pci.slot, pci.function))
            else:
                usb = hd.host_device
                hdev.append("<hostdev mode='subsystem' type='usb' managed='%s'>" % managed)
                src.append("<source>")
                data.append("<vendor id='0x%s' product id='0x%s'/>" % (usb.vendor_id, usb.product_id))
            src.append(data)
            src.append("</source>")
            hdev.append(src)
            hdev.append("</hostdev>")
            host_devices.append(hdev)
        return host_devices

    def _get_sound_xml(self):
        ret = ''
        if self.options.sound and self.options.sound != 'none':
            ret = ["<sound model='%s'/>" % self.options.sound]
        return ret

    def _get_keymap_xml(self):
        ret = []
        keymap = keytable.get_keymap(self.options.keymap)
        if keymap != None:
            ret = ['<keymap>%s</keymap>' % keymap]
        return ret

    def _get_input_xml(self):
        """Get the input config in the libvirt XML format."""
        if self.options.graphics == 'none':
            return []
        if self.options.usb_mouse_abs:
            ret = "<input type='tablet' bus='usb'/>"
        else:
            ret = "<input type='mouse' bus='ps2'/>"
        return [ret]

    def _get_graphics_xml(self):
        """Get the graphics config in the libvirt XML format."""
        graphics = []
        if self.options.graphics == 'none':
            if caps.kvm_arch() == 's390x':
                graphics.append("<console type='pty'>")
                pts = ["<source path='/dev/pts/1'/>"]
                graphics.append(pts)
                target = ["<target type='sclp' port='0'/>"]
                graphics.append(target)
                graphics.append("</console>")
            return graphics
        if self.options.graphics == 'vesa':
            graphics.append("<video>")
            model = ["<model type='vga'/>"]
            graphics.append(model)
            graphics.append("</video>")
        ret = ''
        if self.options.graphics_viewer == 'vnc' or self.options.graphics_viewer == 'virt-viewer':
            ret = "<graphics type='vnc'"
            if self.options.vncport:
                ret += " port='%d'" %(self.options.vncport,)
            else:
                ret += " port='-1' autoport='yes'"
            if self.options.vnc_password:
                ret += " password='%s'" %(self.options.vnc_password,)
            if self.options.keymap:
                ret += " keymap='%s'" %(self.options.keymap,)
            ret += '/>'
        elif self.options.graphics_viewer == 'sdl':
            ret = "<graphics type='sdl'/>"
        graphics.append(ret)
        return graphics

    def _get_misc_xml(self):
        ret = []
        if self.options.os_type == 'oes2nw':
            ret.append("<memballoon model='none'/>")
            ret.append("<controller type='usb' index='0' model='none'/>")
        return ret

    def GetConfigXml(self):
        raise RuntimeError('pure virtual')

    def PreInstall(self, progress=lambda f:None):
        if hypervisor.connection is None:
            hypervisor.connection = libvirt_hypervisor.LibVirtHypervisor("qemu:///system")
        if hypervisor.connection.domain_running(self.options.name):
            raise VmError(VmError.E_VM_EXISTS)
        if self.options.direct_kernel_boot and not self.options.pxe_boot:
            if self.options.os_settings:
                self.os().actions().setupOsSettings(self, lambda f:progress(f*0.05))
            self.dkb = DirectKernelBoot(self.options)
            self.dkb._cleanup_kernel()  # In case the install was retried
            (self.kernel, self.initrd) = self.dkb._get_directkernelboot_install_images(lambda f:progress(0.5+f*0.5))
        self.create_devices(lambda f:progress(0.05 + f*0.95))

    def Start(self):
        def indent(nodes, depth=0):
            def process(node):
                if type(node) is str:
                    return '  '*depth+node
                return indent(node, depth+1)
            return '\n'.join(list(map(process, nodes)))
        if (hypervisor.connection.domain_running(self.options.name) or
            hypervisor.connection.domain_running(self.options.uuid)):
            raise VmError(VmError.E_VM_EXISTS)
        # If there is an existing configuration by this name it is not running,
        # but it may have a different (conflicting) UUID, so delete it.
        hypervisor.connection.delete(self.options.name)
        try:
            domXML = indent(self.GetConfigXml())
            try:
                virDom = hypervisor.connection.getconn().defineXML(domXML)
            except:
                # Retry with a new connection should the defineXML call fail
                virConn = hypervisor.connection.reset_connection()
                virDom = hypervisor.connection.getconn().defineXML(domXML)
            virDom.create()
        except RuntimeError as e:
            log.info("domain create failed")
            self.Stop()
            hypervisor.connection.delete(self.options.name)
            raise VmError(VmError.E_NO_START, details=e.args[0])
        if not hypervisor.connection.wait_domain(self.options.name):
            log.info("Domain failed to start")
            self.Stop()
            hypervisor.connection.delete(self.options.name)
            raise VmError(VmError.E_NO_START)
        self.uuid = self.options.uuid

    def Stop(self):
        # Slightly safer to stop by running-uuid, if it exists, rather than name.
        name = self.uuid
        if self.options.direct_kernel_boot and self.dkb:
            self.dkb._cleanup_kernel()
            self.dkb = None
        if not name:
            name = self.options.name
        try:
            hypervisor.connection.destroy(name)
        except:
            pass
        self.uuid = None

    def PostInstallConf(self):
        """Perform any post-install cleanup and configuration."""
        self.isInstall = False
        self._cleanup_tmpdisks()
        self.os().actions().postInstall(self)

    def manage_disks(self):
        dms = vmdisks.diskmap.DiskMapStatic()
        for disk in self.options.disks:
            dms.manage_disk(disk)
        dms.flush()

    def unmanage_disks(self):
        dms = vmdisks.diskmap.DiskMapStatic()
        for disk in self.options.disks:
            try:
                dms.unmanage_disk(disk)
            except:
                # Possible that someone rm'd the disk list file...
                pass
        dms.flush()

    def start_from_disk(self):
        """Restart the guest from its disks."""
        def indent(nodes, depth=0):
            def process(node):
                if type(node) is str:
                    return '  '*depth+node
                return indent(node, depth+1)
            return '\n'.join(list(map(process, nodes)))
        self.create_devices()
        # Ensure that the install settings are removed; overlaying settings
        # might break (specifically in the case of kernel vs. bootloader).
        # we need to delay long enough to ensure the networking scripts don't operate on the wrong interface
        # since the old and new tap network interfaces have the same names.
        time.sleep(2)
        hypervisor.connection.delete(self.options.name)
        try:
            virDom = hypervisor.connection.getconn().defineXML(indent(self.GetConfigXml()))
            virDom.create()
        except RuntimeError as e:
            log.info(" domain create failed")
            self.Stop()
            hypervisor.connection.delete(self.options.name)
            raise VmError(VmError.E_NO_START, details=e.args[0])
        if not hypervisor.connection.wait_domain(self.options.name):
            log.info("Domain failed to start")
            self.Stop()
            hypervisor.connection.delete(self.options.name)
            raise VmError(VmError.E_NO_START)
        self.uuid = self.options.uuid

    def create_devices(self, progress=lambda f:None):
        disks = self.options.disks + self.tmpdisks
        times = [d.setup_time() for d in disks]
        cum_times = [0.0]
        for i in range(len(times)):
            cum_times.append(cum_times[-1]+times[i])
        for i in range(len(disks)):
            disks[i].setup(lambda f: progress(cum_times[i]/cum_times[-1] + f*cum_times[i+1]/cum_times[-1]))

    def prepare_host_devices_for_passthrough(self):
        log.debug("KVMGuest.prepare_host_devices_for_passthrough: libvirt should handle this operation")

    def FailedInstall(self):
        log.info("Installation failed, so destroying VM")
        self.Stop()
        hypervisor.connection.delete(self.options.name)
        if self.options.preserve_on_error:
            log.info("Installation failed, not cleaning up disks because preserve_on_error is set")
        else:
            log.info("Installation failed, so cleaning up disks")
            self._cleanup_tmpdisks()
            self.unmanage_disks()
            for disk in self.options.disks:
                disk.reset()

    def _cleanup_tmpdisks(self):
        for disk in self.tmpdisks:
            try:
                os.unlink(disk.get_path())
            except:
                pass
        self.tmpdisks = []

    def __del__(self):
        log.debug("KVMGuest.__del__")
        self._cleanup_tmpdisks()

