# Utility functions used for guest installation
#
# Initially derived from virt-inst, Copyright 2005-2006 Red Hat, Inc.
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 

import os
import random
import re
from subprocess import *
from functools import reduce
import tempfile
import subprocess

from .log import log

_host_repo_url = None

def run(args):
    try:
        log.debug("Running: %s" % str(args))
        p = Popen(args=args, stdin=None, stdout=PIPE, stderr=STDOUT, close_fds=True,
                  env={'PATH': '/sbin:/usr/sbin:/bin:/usr/bin', 'TERM':'vt100'})
        (pid, status) = os.waitpid(p.pid, 0)
        output = p.stdout.read()
        if status:
            raise RuntimeError(output)
    except OSError as e:
        raise RuntimeError(str(e))
    if output is not None:
        output = output.decode('utf-8')
    return output

def run_zypper(cmd):
    zypper_cmd = "/usr/bin/zypper " + cmd
    try:
        p = subprocess.Popen(zypper_cmd, stdin=subprocess.PIPE, stdout=PIPE, stderr=STDOUT, shell=True)
        (pid, status) = os.waitpid(p.pid, 0)
        zypper_out = p.stdout.read()
        return zypper_out.decode('utf-8')
    except Exception as e:
        return ""


def close_fds(fds=range(0,256)):
    for fd in fds:
        try:
            os.close(fd)
        except OSError:
            pass
    if 0 in fds:
        os.open('/dev/null', os.O_RDWR)
    if 1 in fds:
        os.open('/dev/null', os.O_RDWR)
    if 2 in fds:
        os.open('/dev/null', os.O_RDWR)

def daemonize(fds=range(0,256), cb=None):
    pid = os.fork()
    if pid == 0:
        os.setsid()
        pid = os.fork()
        if pid == 0:
            os.chdir('/')
            os.umask(0o22)
            if cb:
                cb()
            close_fds(fds)
            return True
        else:
            os._exit(0)
    else:
        return False

# Word-wraps the text, and handles unicode.
# See http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/148061
# python3 rx=re.compile("([\u2e80-\uffff])", re.UNICODE)
rx=re.compile(u"([\u2e80-\uffff])", re.UNICODE)
def wrap(text, width=79, encoding="utf-8"):
    return reduce(lambda line, word, width=width: '%s%s%s' %
                (line,
                 [' ','\n', ''][(len(line)-line.rfind('\n')-1
                       + len(word.split('\n',1)[0] ) >= width) or
                      line[-1:] == '\0' and 2],
                 word),
                rx.sub(r'\1\0 ', str(text)).split(' ')
            ).replace('\0', '')

# Numeric-aware sorting of strings
# Inspired by http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/135435
def _generate_index(str):
    """Splits a string into alpha and numeric elements, which
    is used as an index for sorting"""
    index = []
    def _append(fragment, alist=index):
        if fragment.isdigit():
            fragment = int(fragment)
        alist.append(fragment)
    prev_isdigit = str[0].isdigit()
    current_fragment = ''
    for char in str:
        curr_isdigit = char.isdigit()
        if curr_isdigit == prev_isdigit:
            current_fragment += char
        else:
            _append(current_fragment)
            current_fragment = char
            prev_isdigit = curr_isdigit
    _append(current_fragment)    
    return tuple(index)

def sort_tuples_by_nth(alist, nth=0):
    indices = list(map(_generate_index, [t[nth] for t in alist]))
    decorated = list(zip(indices, alist))
    decorated.sort()
    return [item[1] for item in decorated]

def sort_numeric_aware(alist):
    indices = list(map(_generate_index, alist))
    decorated = list(zip(indices, alist))
    decorated.sort()
    return [item for index, item in decorated]

def sort_dict_keys_by_value(d):
    sorted = sort_tuples_by_nth(list(d.items()), 1)
    return [ item[0] for item in sorted ]

def filtered_progress(fn, step=0.01):
    """Returns a function which takes a fraction (0.0 .. 1.0) describing
    progress.  When the progress advances beyond a specified step size,
    the function calls 'fn' with the current progress."""
    def p(f):
        if f < p.f + step:
            return
        p.f = f
        fn(p.f)
    p.f = 0.0
    fn(p.f)
    return p

def copy_to_temp(fileobj, prefix, progress=lambda f:None):
    (fd, fn) = tempfile.mkstemp(prefix=prefix)
    log.debug("Copying to temporary file '%s'" % fn)
    read = size = 0
    try:
        size = os.fstat(fileobj.fileno()).st_size
    except:
        pass
    block_size = 65536
    while 1:
        try:
            buff = fileobj.read(block_size)
            if not buff:
                break
            os.write(fd, buff)
        except:
            os.close(fd)
            os.unlink(fn)
            raise
        read += len(buff)
        if read <= size:  # Some file objects can't accurately report size
            progress(float(read) / float(size))
    os.close(fd)
    return fn

def copy_to_bak(fname):
    fn = fname + "._bak"
    fo = open(fname)
    fb = open(fn, "w+")
    buf = fo.read()
    fb.write(buf)
    fo.close()
    fb.close()
    return fn

def mke2fsImage(filename, progress=lambda f:None):
    args = ['/sbin/mkfs.ext2', '-F', '-m', '0', '-q', '--', filename]
    log.debug("Creating filesystem: %s" % str(args))
    run(args)
    progress(1.0)

def mkdosfsImage(filename, progress=lambda f:None):
    args = ['/sbin/mkdosfs', '--', filename]
    log.debug("Creating filesystem: %s" % str(args))
    run(args)
    progress(1.0)

# the following three functions are from xend/uuid.py and are thus
# available under the LGPL,
# Copyright 2005 Mike Wray <mike.wray@hp.com>
# Copyright 2005 XenSource Ltd
def randomUUID():
    """Generate a random UUID."""
    return [ random.randint(0, 255) for _ in range(0, 16) ]

def uuidToString(u):
    return '-'.join(['%02x' * 4, '%02x' * 2, '%02x' * 2, '%02x' * 2,
                     '%02x' * 6]) % tuple(u)

def uuidFromString(s):
    s = s.replace('-', '')
    return [ int(s[i : i + 2], 16) for i in range(0, 32, 2) ]

def addInstallSourceRepo(source=None):
    if not source or type(source) is not str:
        return
    if source.startswith('ftp://') or source.startswith('http://') or source.startswith('nfs://') or source.startswith('smb://'):
        found = 0
        locations = run_zypper("lr -u | awk -F'|' '{ print $6 }'")
        locations = locations[(locations.rfind('URI')):].split()
        for loc in locations:
            if loc == source:
                found = 1
                break
        if not found:
            names = run_zypper("lr -u | awk -F'|' '{ print $3 }'")
            names = names[(names.rfind('Name')):].split()
            index = 1
            repo_name = "Xen-" + str(index)
            for nam in names:
                if nam == repo_name:
                    index += 1
                    repo_name = "Xen-" + str(index)
            run_zypper("ar -d -C %s %s" % (source, repo_name))

def getDom0InstallSource():
    global _host_repo_url
    if _host_repo_url is not None:
        return _host_repo_url

    if os.path.exists('/var/lib/YaST2/install.inf'):
        server_ip = server_name = server_dir = inst_mode = None
        f = open('/var/lib/YaST2/install.inf')
        lines = f.readlines()
        f.close()
        # Newer install.inf files use RepoURL.  Older versions require parsing more fields
        for line in lines:
            if line.startswith('RepoURL:'):
                repo_url = line[:-1].split('?', 1)[0]
                repo_url = repo_url.split(' ')
                if len(repo_url) > 1:
                    if repo_url[1].startswith('ftp:') or repo_url[1].startswith('http:') or \
                       repo_url[1].startswith('smb:') or repo_url[1].startswith('nfs:'):
                        _host_repo_url = repo_url[1]
                        return repo_url[1]
                return None
            elif line.startswith('InstMode:'):
                inst_mode = line[:-1].split('?', 1)[0]
                inst_mode = inst_mode.split(' ')
                inst_mode = inst_mode[1]
                if inst_mode != 'ftp' and inst_mode != 'http' and \
                   inst_mode != 'smb' and inst_mode != 'nfs':
                    return None
            elif line.startswith('ServerIP:'):
                server_ip = line[:-1].split('?', 1)[0]
                server_ip = server_ip.split(' ')
                server_ip = server_ip[1]
            elif line.startswith('ServerName:'):
                server_name = line[:-1].split('?', 1)[0]
                server_name = server_name.split(' ')
                server_name = server_name[1]
            elif line.startswith('Serverdir:'):
                server_dir = line[:-1].split('?', 1)[0]
                server_dir = server_dir.split(' ')
                server_dir = server_dir[1]
        if inst_mode:
            repo_url = inst_mode + "://"
            if server_name:
                repo_url = repo_url + server_name + "/"
                if server_dir:
                    repo_url = repo_url + server_dir
            elif server_ip:
                repo_url = repo_url + server_ip + "/"
                if server_dir:
                    repo_url = repo_url + server_dir
            _host_repo_url = repo_url
            return repo_url
    else:
        (_,zypper_output) = lookupZypperRepos()
        if len(zypper_output):
            _host_repo_url = zypper_output[0]
            return _host_repo_url
    return None

def lookupZypperRepos(dom0_inst_source=None):
    try:
        zypper_output = run_zypper("lr -u -E -P")
        zypper_list = zypper_output.split("\n")
        zypper_header = [x.strip(' ') for x in zypper_list[0].split("|")]
        uri_index = zypper_header.index("URI")
    except Exception as e:
        if dom0_inst_source is None:
            dom0_inst_source = []
        if type(dom0_inst_source) is str:
            dom0_inst_source = [dom0_inst_source]
        return (0, dom0_inst_source)

    index_dom0 = -1
    number_of_sources = 0
    zypper_output = []
    for repo in zypper_list:
        repo = [x.strip(' ') for x in repo.split("|")]
        if len(repo) >= uri_index:
            uri_str = repo[uri_index]
            if uri_str.startswith('ftp://') or uri_str.startswith('http://') or \
               uri_str.startswith('nfs://') or uri_str.startswith('smb://'):
                zypper_output.append(uri_str)
                if dom0_inst_source is not None and uri_str == dom0_inst_source:
                    index_dom0 = number_of_sources
                number_of_sources += 1

    if index_dom0 == -1:
        if dom0_inst_source:
            index_dom0 = 0
            zypper_output.insert(0, dom0_inst_source)
        elif len(zypper_output):
            index_dom0 = 0
            dom0_inst_source = zypper_output[0]
    return (index_dom0, zypper_output)

def getInstallSources():
    return lookupZypperRepos(getDom0InstallSource())

def get_running_vms():
    managed_vms = []
    use_virsh = False
    if is_xend_running():
        rfd = os.popen("/usr/sbin/xm list")
    else:
        rfd = os.popen("/usr/bin/virsh list --all --name")
        use_virsh = True
    data = rfd.read()
    status = rfd.close()
    if not use_virsh:
        lines = data.splitlines()[1:]
        lines.pop(0)
    else:
        lines = data.splitlines()
    for line in lines:
        if line is '' or line[0] in (' ', '\t'):
            continue
        words = line.split()
        managed_vms.append(words[0])
    return managed_vms

def get_pv_vms():
    managed_pv_vms = []
    managed_vms = get_running_vms()
    use_xend = False
    for vm in managed_vms:
        if vm == "Domain-0" or vm == "Xenstore":
            continue
        found_pv = True
        if is_xend_running():
            use_xend = True
            output = run(['/usr/sbin/xm', 'list', '-l', vm])
        else:
            output = run(['/usr/bin/virsh', 'dumpxml', vm])
        if output:
            output = output.split('\n')
            for line in output:
                if use_xend is True:
                    # Check for HVM footprint
                    # 1. Has '(hvm' field
                    # 2. Has a '(loader ' string - usually set to hvmloader
                    if "(hvm" in line:
                        found_pv = False
                        break
                    if "(loader" in line and 'hvmloader' in line:
                        found_pv = False
                        break
                else:
                    # Check for HVM footprint
                    # 1. machine='xenfv'>hvm
                    # 2. <loader type='rom'>/usr/lib/xen/boot/hvmloader</loader>
                    if "machine='xenfv'" in line:
                        found_pv = False
                        break
                    if "<loader type=" in line and "hvmloader" in line:
                        found_pv = False
                        break
        if found_pv:
            managed_pv_vms.append(vm)
    return managed_pv_vms

_xend_running = None

def is_xend_running():
    global _xend_running
    if _xend_running is not None:
        return _xend_running
    if os.path.exists('/usr/sbin/xend'):
        try:
            run(['/usr/sbin/xend', 'status'])
            _xend_running = True
            return True
        except:
            pass
    _xend_running = False
    return False

def get_config_file_option(config_file, var):
    if config_file is None or not os.path.exists(config_file):
        return None
    f = open(config_file)
    lines = f.readlines()
    f.close()
    for line in lines:
        if not line.startswith(var):
            continue
        value = line[:-1].split('=', 1)[1]
        value = value.strip('"')
        value = value.strip('\'')
        return value
    return None

def read_config_file(vm_name, generate_managed_config, backup):
    # Caller must delete temporary files when done
    log.debug('Reading configuration file, %s, data = %s' % (vm_name,generate_managed_config))
    bname = None
    if generate_managed_config:
        (fd, fname) = tempfile.mkstemp(prefix=vm_name + '.')
        if is_xend_running():
            os.system("/usr/sbin/xm list -l %s > %s" % (vm_name, fname))
        else:
            os.system("/usr/bin/virsh dumpxml %s > %s" % (vm_name, fname))
        if not os.path.exists(fname):
            return (None, None, None)
        if not os.stat(fname).st_size:
            os.unlink(fname)
            return (None, None, None)
        if backup:
            f = open(fname)
            bname = copy_to_bak(fname)
            f.close()
    else:
        f = open(vm_name)
        fname = copy_to_temp(f,vm_name + '.')
        if backup:
            bname = copy_to_bak(fname)
        f.close()
    f = open(fname)
    lines = f.readlines()
    f.close()
    return (fname, lines, bname)
 
def lookup_sxp_value(data, var):
    var_start = 0
    var_data = ""
    for line in data:
        line = line.strip(" \n")
        if line.startswith("(" + var):
            var_start = 1
            var_data = line
            if line.endswith(")"):
                var_start -= 1
                break
            continue
        if var_start > 0:
            var_data += line
            if line.startswith("("):
                var_start += 1
            if line.endswith(")"):
                var_start -= 1
            if var_start:
                continue
    if len(var_data):
        value = var_data.strip(" )(\n")
        value = value.lstrip(var)
        value = value.strip("\' ")
        if len(value):
            return value
    return None

