# Common code for all Xen guests
#
# Initially derived from virt-inst, Copyright 2005-2006 Red Hat, Inc.
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA 

import os

from .exceptions import *
from .globals import *
from .log import log
from . import paths
from . import util
import vmdisks
from . import hypervisor
from . import xen_hypervisor
from . import xl_hypervisor
from . import libvirt_hypervisor
from . import keytable
from .guest import Guest
from .direct_kernel_boot import DirectKernelBoot

class XenGuest(Guest):
    def _get_name_xen(self):
        return 'name="%s"\n' % self.options.name

    def _get_description(self):
        return 'description="%s"\n' % self.options.description

    def _get_description_xml(self):
        return ['<description>%s</description>' % self.options.description]

    def _get_memory_xml(self):
        # XML wants KB
        return ['<memory>%d</memory>' % (self.options.max_memoryMB*1024,),
                '<currentMemory>%d</currentMemory>' % (self.options.memoryMB*1024,)]
    def _get_memory_xen(self):
        # config file wants MB
        return 'memory=%d\n' % self.options.memoryMB
    def _get_max_memory_xen(self):
        # config file wants MB
        return 'maxmem=%d\n' % self.options.max_memoryMB

    def _get_uuid_xml(self):
        return ['<uuid>%s</uuid>' % self.options.uuid]
    def _get_uuid_xen(self):
        return 'uuid="%s"\n' % self.options.uuid

    def _get_action(self, action):
        if self.isInstall:
            return 'destroy'
        else:
            if action == 'crash': return self.options.on_crash
            if action == 'poweroff': return self.options.on_poweroff
            if action == 'reboot': return self.options.on_reboot
    def _get_action_xml(self):
        return ['<on_poweroff>%s</on_poweroff>' % self._get_action('poweroff'),
                '<on_reboot>%s</on_reboot>' % self._get_action('reboot'),
                '<on_crash>%s</on_crash>' % self._get_action('crash')]
    def _get_action_xen(self):
        return 'on_poweroff="%s"\non_reboot="%s"\non_crash="%s"\n' % \
               (self._get_action('poweroff'), self._get_action('reboot'), self._get_action('crash'))

    def _get_vcpus_xml(self):
        return ['<vcpu>%d</vcpu>' % self.options.vcpus]
    def _get_vcpus_xen(self):
        return 'vcpus=%d\n' % self.options.vcpus

    def _get_localtime_xml(self):
        if self.os().localtime():
            return ["<clock sync='localtime'/>"]
        return ["<clock sync='utc'/>"]
    def _get_localtime_xen(self):
        if self.os().localtime():
            return 'localtime=1\n'
        return 'localtime=0\n'

    def getCommonXen(self):
        s = ''
        s += self._get_name_xen()
        s += self._get_description()
        s += self._get_uuid_xen()
        s += self._get_memory_xen()
        s += self._get_max_memory_xen()
        s += self._get_vcpus_xen()
        s += self._get_action_xen()
        s += self._get_localtime_xen()
        s += self._get_keymap_xen()
        return s

    def getCommonXml(self):
        s = []
        s.extend(self._get_name_xml())
        s.extend(self._get_description_xml())
        s.extend(self._get_uuid_xml())
        s.extend(self._get_memory_xml())
        s.extend(self._get_vcpus_xml())
        s.extend(self._get_action_xml())
        s.extend(self._get_localtime_xml())
        s.extend(self._get_keymap_xml())
        return s

    def _get_disk_xml(self):
        """Get the disk config in the libvirt XML format"""
        ret = []
        saved_mode = None
        target_disk = self.options.disks[0]
        if self.isInstall and target_disk:
            index, saved_mode = target_disk.get_cache_mode()
            # The destination disk only during installation is this considered safe to do
            target_disk.set_cache_mode("unsafe")
        for disk in self.options.disks + self.tmpdisks:
            ret.extend(disk.get_xml_config())
        if self.isInstall and target_disk:
            target_disk.set_cache_mode(saved_mode)
        return ret

    def _get_disk_xen(self):
        """Get the disk config in the xend python format"""
        ret = 'disk=[ '
        saved_mode = None
        target_disk = self.options.disks[0]
        if self.isInstall and target_disk:
            index, saved_mode = target_disk.get_cache_mode()
            # The destination disk only during installation is this considered safe to do
            target_disk.set_cache_mode("unsafe")
        for disk in self.options.disks + self.tmpdisks:
            ret += disk.get_xen_config()
            ret += ', '
        ret += ']'
        if self.isInstall and target_disk:
            target_disk.set_cache_mode(saved_mode)
        return ret

    def _get_network_xml(self):
        """Get the network config in the libvirt XML format"""
        nics = []
        need_emulated_nic = False
        if self.options.full_virt and self.options.pxe_boot:
            # If PXE booting, must have emulated nic
            need_emulated_nic = True
            for nic in self.options.nics:
                if nic.model != 'para':
                    need_emulated_nic = False
        for n in self.options.nics:
            model = ""
            nic = ["<interface type='bridge'>"]
            opts = []
            if self.options.full_virt:
                if self.isInstall and need_emulated_nic:
                    opts.append("<model type='rtl8139'/>")
                elif n.model and n.model != 'para':
                    opts.append("<model type='%s'/>" % n.model)
            if n.bridge:
                opts.append("<source bridge='%s'/>" % n.bridge)
            if n.mac:
                opts.append("<mac address='%s'/>" % n.mac)
            opts.append("<script path='/etc/xen/scripts/vif-bridge'/>")
            nic.append(opts)
            nic.append('</interface>')
            nics.append(nic)
        return nics

    def _get_network_xen(self):
        """Get the network config in the xend python format"""
        need_emulated_nic = False
        if self.options.full_virt and self.options.pxe_boot:
            # If PXE booting, must have emulated nic
            need_emulated_nic = True
            for nic in self.options.nics:
                if nic.model != 'para':
                    need_emulated_nic = False
        ret = 'vif=[ '
        for nic in self.options.nics:
            ret += "'"
            needComma = 0
            commas = [ '', ',' ]
            if nic.mac:
                ret += commas[needComma]
                ret += 'mac=%s' % (nic.mac,)
                needComma = 1
            if nic.bridge:
                ret += commas[needComma]
                ret += 'bridge=%s' % ( nic.bridge, )
                needComma = 1
            if self.options.full_virt:
                if nic.model == 'para':
                    ret += commas[needComma]
                    if self.isInstall and need_emulated_nic:
                        ret += 'model=rtl8139'
                    else:
                        ret += 'type=netfront'
                    needComma = 1
                else:
                    if nic.model:
                        ret += commas[needComma]
                        ret += 'model=%s' % ( nic.model, )
                        needComma = 1
                    if not self.isInstall:
                        if self.options.os_type == 'sles15' or self.options.os_type == 'sled15' or \
                           self.options.os_type == 'sles12' or self.options.os_type == 'sled12' or \
                           self.options.os_type == 'sles11' or self.options.os_type == 'sled11':
                            ret += commas[needComma]
                            ret += 'type=netfront'
                            needComma = 1
            ret += "', "
        ret += ']'
        return ret

    def _get_serial_xml(self):
        """Get the serial config in the libvirt XML format"""
        serial = []
        if self.options.graphics == 'none':
            serial.append("<serial type='pty'>")
            target = ["<target port='0'/>"]
            serial.append(target)
            serial.append("</serial>")
            serial.append("<console type='pty'>")
            target = ["<target type='serial' port='0'/>"]
            serial.append(target)
            serial.append("</console>")
        return serial

    def _get_host_devices_xen(self):
        """Get the host device config in the xend python format"""
        # Example: pci=[ '0000:00:1c.4,managed=1' ]
        # Example: usbdevice="host:413c:2002"
        if not self.options.host_devices:
            return ""
        pci = 'pci=[ '
        usb = 'usbdevice=[ '
        found_usb = False
        found_pci = False
        for hd in self.options.host_devices:
            dev = hd.host_device
            if hd.type == 'pci':
                if found_pci:
                    pci += ', '
                found_pci = True
                if hd.managed:
                    pci += "'%s:%s:%s.%s,managed=1'" % (dev.domain, dev.bus, dev.slot, dev.function)
                else:
                    pci += "'%s:%s:%s.%s'" % (dev.domain, dev.bus, dev.slot, dev.function)
            else:
                if found_usb:
                    usb += ', '
                found_usb = True
                usb += "'host:%s:%s'" % (dev.vendor_id, dev.product_id)
        pci += ' ]'
        ret = pci
        if found_usb is True:
            usb += ' ]'
            ret = pci + '\n' + usb
        return ret

    def _get_host_devices_xml(self):
        """Get the host device config in the libvirt XML format"""
        ##### Example PCI
        #<hostdev mode='subsystem' type='pci' managed='no'>
        #  <source>
        #    <address domain='0x0000' bus='0x06' slot='0x03' function='0x0'/>
        #  </source>
        #</hostdev>
        ##### Example USB
        #<hostdev mode="subsystem" type="usb" managed="no">
        #  <source>
        #    <vendor id="0x04f3"/>
        #    <product id="0x0213"/>
        #  </source>
        #</hostdev>
        if not self.options.host_devices:
            return ""
        host_devices = []
        for hd in self.options.host_devices:
            hdev = []
            src = []
            data = []
            managed = 'no'
            if hd.managed:
                managed = 'yes'
            if hd.type == 'pci':
                pci = hd.host_device
                hdev.append("<hostdev mode='subsystem' type='pci' managed='%s'>" % managed)
                src.append("<source>")
                data.append("<address domain='0x%s' bus='0x%s' slot='0x%s' function='0x%s'/>" % (pci.domain, pci.bus, pci.slot, pci.function))
            else:
                usb = hd.host_device
                hdev.append("<hostdev mode='subsystem' type='usb' managed='%s'>" % managed)
                src.append("<source>")
                data.append("<vendor id='0x%s' product id='0x%s'/>" % (usb.vendor_id, usb.product_id))
            src.append(data)
            src.append("</source>")
            hdev.append(src)
            hdev.append("</hostdev>")
            host_devices.append(hdev)
        return host_devices

    def _get_cpuid_xen(self):
        """Get the cpuid config in the xend python format"""
        cpuid_len = len(self.options.cpuid)
        if cpuid_len == 0:
            return ""
        firsttime = 1
        padding = '            '
        ret = 'cpuid = [ '
        for cpuid in self.options.cpuid:
            if not firsttime:
                flen = len(cpuid.get_function()) + 1
                if flen > 11:
                    flen = 11
                pad = padding[flen:]
                if cpuid_len > 0:
                    ret += ",\n" + pad
            else:
                firsttime = 0
            ret += cpuid.get_formatted()
            cpuid_len -= 1
        ret += "]"
        return ret

    def _get_cpu_check_xen(self):
        """Get the cpu_check config in the xend python format"""
        cpu_check_len = len(self.options.cpu_check)
        if cpu_check_len == 0:
            return ""
        firsttime = 1
        padding = '            '
        ret = 'cpu_check = [ '
        for cpu_check in self.options.cpu_check:
            if not firsttime:
                flen = len(cpu_check.get_function()) + 1
                if flen > 11:
                    flen = 11
                pad = padding[flen:]
                if cpu_check_len > 0:
                    ret += ",\n" + pad
            else:
                firsttime = 0
            ret += cpu_check.get_formatted()
            cpu_check_len -= 1
        ret += "]"
        return ret

    def _get_sound_xml(self):
        ret = ''
        if self.options.sound and self.options.sound != 'none':
            ret = ["<sound model='%s'/>" % self.options.sound]
        return ret

    def _get_sound_xen(self):
        ret = ''
        if self.options.sound and self.options.sound != 'none':
            ret = "soundhw=\"%s\"\n" %(self.options.sound)
        return ret

    def _get_keymap_xml(self):
        ret = []
        #keymap = keytable.get_keymap(self.options.keymap)
        #if keymap != None:
        #    ret = ['<keymap>%s</keymap>' % keymap]
        return ret

    def _get_keymap_xen(self):
        ret = ''
        keymap = keytable.get_keymap(self.options.keymap)
        if keymap != None:
            ret = "keymap=\"%s\"\n" %(keymap)
        return ret

    def _get_graphics_xml(self):
        """Get the graphics config in the libvirt XML format."""
        if self.options.graphics == 'none':
            return []
        ret = ''
        if self.options.graphics_viewer == 'vnc' or self.options.graphics_viewer == 'virt-viewer':
            ret = "<graphics type='vnc'"
            if self.options.vncport:
                ret += " port='%d'" %(self.options.vncport,)
            if self.options.vnc_password:
                ret += " password='%s'" %(self.options.vnc_password,)
            keymap = keytable.get_keymap(self.options.keymap)
            if keymap != None:
                ret += " keymap='%s'" %(keymap,)
            ret += '/>'
        elif self.options.graphics_viewer == 'sdl':
            ret = "<graphics type='sdl'/>"
        return [ret]

    def _get_graphics_xen(self):
        """Get the graphics config in the xend python format"""
        if self.options.graphics == 'none':
            return 'nographic=1'

        lines = []
        options = []
        viewer = self.options.graphics_viewer
        if viewer == 'vnc' or viewer == 'virt-viewer':
            if self.options.vnc_password:
                options.append("vncpasswd='%s'" % self.options.vnc_password)
            port = self.options.vncport
            if port is None:
                options.append('vncunused=1')
            elif port >= 5900:
                options.append('vncdisplay=%d' % (port - 5900,))
                options.append('vncunused=0')
            viewer = 'vnc'
        else:
            assert(viewer == 'sdl')
            options=[]

        if self.options.graphics == 'para':
            options = ["".join(s.split("'")) for s in options]
            lines.append('vfb=[\'type=%s,%s\']' % (viewer, ','.join(options)))
        else:
            stdvga = 0
            if self.options.graphics == 'vesa':
                stdvga=1
            lines.append('stdvga=%d' % stdvga)
            lines.append('%s=1' % viewer)
            lines += options

        return '\n'.join(lines)

    def GetConfigXml(self):
        raise RuntimeError('pure virtual')
    def GetConfigXen(self):
        raise RuntimeError('pure virtual')

    def PreInstall(self, progress=lambda f:None):
        if hypervisor.connection is None:
            if self.options.use_libvirt:
                hypervisor.connection = libvirt_hypervisor.LibVirtHypervisor("xen:///")
            elif self.options.use_xl or not util.is_xend_running():
                hypervisor.connection = xl_hypervisor.Xl(None)
            else:
                hypervisor.connection = xen_hypervisor.Xend(None)
        if hypervisor.connection.domain_running(self.options.name):
            raise VmError(VmError.E_VM_EXISTS)
        if not self.options.full_virt or self.options.direct_kernel_boot:
            if self.options.os_settings:
                self.os().actions().setupOsSettings(self, lambda f:progress(f*0.05))
            if self.options.direct_kernel_boot and not self.options.pxe_boot:
                self.dkb = DirectKernelBoot(self.options)
                self.dkb._cleanup_kernel()  # In case the install was retried
                (self.kernel, self.initrd) = self.dkb._get_directkernelboot_install_images(lambda f:progress(0.5+f*0.5))
        self.create_devices(lambda f:progress(0.05 + f*0.95))

    def Start(self):
        def indent(nodes, depth=0):
            def process(node):
                if type(node) is str:
                    return '  '*depth+node
                return indent(node, depth+1)
            return '\n'.join(list(map(process, nodes)))
        if (hypervisor.connection.domain_running(self.options.name) or
            hypervisor.connection.domain_running(self.options.uuid)):
            raise VmError(VmError.E_VM_EXISTS)
        # If there is an existing configuration by this name it is not running,
        # but it may have a different (conflicting) UUID, so delete it.
        hypervisor.connection.delete(self.options.name)
        try:
            if self.options.use_libvirt and not self.options.use_xl:
                domXML = indent(self.GetConfigXml())
                try:
                    virDom = hypervisor.connection.getconn().defineXML(domXML)
                except:
                    # Retry with a new connection should the defineXML call fail
                    hypervisor.connection = libvirt_hypervisor.LibVirtHypervisor("xen:///")
                    virDom = hypervisor.connection.getconn().defineXML(domXML)
                virDom.create()
            else:
                is_xm = util.is_xend_running()
                xen_cmd = '/usr/sbin/xm' if (is_xm is True) else '/usr/sbin/xl'
                if self.options.install == OS_INSTALL_UPGRADE:
                    if self.options.upgrade_managed_vm:
                        if is_xm:
                            util.run(['/usr/sbin/xm', 'new', '-F', self.options.upgrade_config])
                            util.run(['/usr/sbin/xm', 'start', self.options.name])
                        else:
                            util.run(['/usr/sbin/xl', 'create', '-f', self.options.upgrade_config])
                    else:
                        util.run([xen_cmd, 'create', '-f', self.options.upgrade_config])
                elif self.options.os_type == 'rear':
                    util.run([xen_cmd, 'create', '-f', os.path.join(self.options.config_dir, self.options.name)])
                else:
                    if is_xm:
                        util.run(['/usr/sbin/xm', 'new', os.path.join(self.options.config_dir, self.options.name)])
                        util.run(['/usr/sbin/xm', 'start', self.options.name])
                    else:
                        util.run(['/usr/sbin/xl', 'create', '-f', os.path.join(self.options.config_dir, self.options.name)])
        except RuntimeError as e:
            log.info("'xm new' or 'xm start' or 'xl create' failed")
            self.Stop()
            hypervisor.connection.delete(self.options.name)
            raise VmError(VmError.E_NO_START, details=e.args[0])
        if not hypervisor.connection.wait_domain(self.options.name):
            log.info("Domain failed to start")
            self.Stop()
            hypervisor.connection.delete(self.options.name)
            raise VmError(VmError.E_NO_START)
        self.uuid = self.options.uuid

    def Stop(self):
        # Slightly safer to stop by running-uuid, if it exists, rather than name.
        name = self.uuid
        if self.options.direct_kernel_boot and self.dkb:
            self.dkb._cleanup_kernel()
            self.dkb = None
        if not name:
            name = self.options.name
        try:
            hypervisor.connection.destroy(name)
        except:
            pass
        self.uuid = None

    def PostInstallConf(self):
        """Perform any post-install cleanup and configuration."""
        self.isInstall = False
        self._cleanup_tmpdisks()
        self.os().actions().postInstall(self)

    def manage_disks(self):
        dms = vmdisks.diskmap.DiskMapStatic()
        for disk in self.options.disks:
            dms.manage_disk(disk)
        dms.flush()

    def unmanage_disks(self):
        dms = vmdisks.diskmap.DiskMapStatic()
        for disk in self.options.disks:
            try:
                dms.unmanage_disk(disk)
            except:
                # Possible that someone rm'd the disk list file...
                pass
        dms.flush()

    def start_from_disk(self):
        """Restart the guest from its disks."""
        def indent(nodes, depth=0):
            def process(node):
                if type(node) is str:
                    return '  '*depth+node
                return indent(node, depth+1)
            return '\n'.join(list(map(process, nodes)))
        self.create_devices()
        # Ensure that the install settings are removed; overlaying settings
        # might break (specifically in the case of kernel vs. bootloader).
        hypervisor.connection.delete(self.options.name)
        try:
            if self.options.use_libvirt:
                virDom = hypervisor.connection.getconn().defineXML(indent(self.GetConfigXml()))
                virDom.create()
            else:
                is_xm = util.is_xend_running()
                xen_cmd = '/usr/sbin/xm' if (is_xm is True) else '/usr/sbin/xl'
                if self.options.install == OS_INSTALL_UPGRADE:
                    if self.options.upgrade_managed_vm:
                        if is_xm:
                            util.run(['/usr/sbin/xm', 'new', '-F', self.options.upgrade_config + "._bak"])
                            util.run(['/usr/sbin/xm', 'start', self.options.name])
                        else:
                            util.run(['/usr/sbin/xl', 'create', '-f', self.options.upgrade_config + "._bak"])
                    else:
                        util.run([xen_cmd, 'create', '-f', self.options.upgrade_config + "._bak"])
                else:
                    if is_xm:
                        util.run(['/usr/sbin/xm', 'new', os.path.join(self.options.config_dir, self.options.name)])
                        util.run(['/usr/sbin/xm', 'start', self.options.name])
                    else:
                        util.run(['/usr/sbin/xl', 'create', '-f', os.path.join(self.options.config_dir, self.options.name)])
        except RuntimeError as e:
            log.info("'xm new' or 'xm start' failed")
            self.Stop()
            hypervisor.connection.delete(self.options.name)
            raise VmError(VmError.E_NO_START, details=e.args[0])
        if not hypervisor.connection.wait_domain(self.options.name):
            log.info("Domain failed to start")
            self.Stop()
            hypervisor.connection.delete(self.options.name)
            raise VmError(VmError.E_NO_START)
        self.uuid = self.options.uuid

    def create_devices(self, progress=lambda f:None):
        disks = self.options.disks + self.tmpdisks
        times = [d.setup_time() for d in disks]
        cum_times = [0.0]
        for i in range(len(times)):
            cum_times.append(cum_times[-1]+times[i])
        for i in range(len(disks)):
            disks[i].setup(lambda f: progress(cum_times[i]/cum_times[-1] + f*cum_times[i+1]/cum_times[-1]))

    def prepare_host_devices_for_passthrough(self):
        ### Xen Only Process
        # 1. Make sure pciback is loaded
        ret = os.system("/sbin/modprobe pciback")
        if ret:
            raise HostDeviceResourceError(HostDeviceResourceError.E_PCIBACK)
        for hd in self.options.host_devices:
            if hd.type == 'pci':
                pci = hd.host_device
                device = "%s:%s:%s.%s" % (pci.domain, pci.bus, pci.slot, pci.function)
                path_to_driver = '/sys/bus/pci/devices/%s/driver' % device
                if os.path.exists(path_to_driver):
                    path_to_driver = os.path.realpath(path_to_driver).replace(" ", "\ ")
                    # 2. Get the driver name for the host device
                    driver = os.path.basename(path_to_driver)
                    # 3. Check to see if driver is already bound to pciback
                    if driver == 'pciback':
                        self.check_for_host_device_in_use(device)
                        continue
                    path_to_pciback_driver = "/sys/bus/pci/drivers/pciback"
                    # 4. Unbind the driver from the device.
                    unbind = path_to_driver + '/unbind'
                    cmd = 'echo -n %s > %s' % (device,unbind)
                    ret = os.system(cmd)
                    if ret:
                        raise HostDeviceResourceError(HostDeviceResourceError.E_UNBIND, details=device)
                    # 5. Give device to 'new_slot' so pciback knows its available
                    new_slot = path_to_pciback_driver + '/new_slot'
                    cmd = 'echo -n %s > %s' % (device,new_slot)
                    ret = os.system(cmd)
                    if ret:
                        raise HostDeviceResourceError(HostDeviceResourceError.E_NEWSLOT, details=device)
                    # 6. Bind the device to pciback
                    bind = path_to_pciback_driver + '/bind'
                    cmd = 'echo -n %s > %s' % (device,bind)
                    ret = os.system(cmd)
                    if ret:
                        raise HostDeviceResourceError(HostDeviceResourceError.E_BIND, details=device)

    def check_for_host_device_in_use(self, device=None):
        ### Xen Only Process
        def lookup_device(device, devices_in_use):
            for d in devices_in_use:
                if d == device:
                    raise HostDeviceResourceError(HostDeviceResourceError.E_IN_USE, details=device)
                    break
        # Get list of managed or running VMs
        managed_vms = util.get_running_vms()
        devices_in_use = []
        for vm in managed_vms:
            if util.is_xend_running():
                output = util.run(['/usr/sbin/xm', 'pci-list', vm])
            else:
                output = util.run(['/usr/sbin/xl', 'pci-list', vm])
            if output:
                output = output.split('\n')
                for line in output:
                    if line != 'Device': 
                        devices_in_use.append(line)
        if device:
            lookup_device(device, devices_in_use)
        else:
            for hd in self.options.host_devices:
                if hd.type == 'pci':
                    pci = hd.host_device
                    device = "%s:%s:%s.%s" % (pci.domain, pci.bus, pci.slot, pci.function)
                    lookup_device(device, devices_in_use)

    def FailedInstall(self):
        log.info("Installation failed, so destroying VM")
        if self.options.install == OS_INSTALL_UPGRADE:
            return
        self.Stop()
        hypervisor.connection.delete(self.options.name)
        if self.options.preserve_on_error:
            log.info("Installation failed, not cleaning up disks because preserve_on_error is set")
        else:
            log.info("Installation failed, so cleaning up disks")
            self._cleanup_tmpdisks()
            self.unmanage_disks()
            for disk in self.options.disks:
                disk.reset()

    def _cleanup_tmpdisks(self):
        for disk in self.tmpdisks:
            try:
                os.unlink(disk.get_path())
            except:
                pass
        self.tmpdisks = []

    def __del__(self):
        log.debug("XenGuest.__del__")
        self._cleanup_tmpdisks()
